using System;
using System.IO;

namespace MP3Cleaner
{
	
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class MP3Cleaner
	{
		private Config MyConfig;
		private GUIStatus MyGUIStatus;
		private bool KeepGoing;

		public MP3Cleaner(Config configuration, GUIStatus status)
		{
			this.MyConfig = configuration;
			this.MyGUIStatus = status;
		}

		public void Run()
		{
			ProccessDir(MyConfig.DirName);		
		}

		public void Stop()
		{
			KeepGoing = false;
		}

		public void ProccessDir(String dirName)
		{
			int result = 0;
			MP3File first;
			MP3File second;
			int totalFiles =0;
			int totalMatches =0;
			int totalTime =0;

			string logfileName = dirName.Substring( 1+dirName.LastIndexOf("\\") );
			logfileName = "output_" + logfileName + ".txt";
			StreamWriter myStream = new StreamWriter(logfileName,false);
			DirectoryInfo dir = new DirectoryInfo(dirName);
			FileInfo[] files = dir.GetFiles("*.mp3");

			MyGUIStatus.SetProgressRange(files.Length);
			totalTime = Environment.TickCount;
			totalFiles = files.Length;

			KeepGoing = true;
			for( int i=0; i< files.Length; i++)
			{
				//make sure the file is valid
				if( null == files[i] )
					continue;

				//construct the first mp3 object
				first = ShellID3TagReader.ReadID3Tags(files[i].FullName);
				
				MyGUIStatus.StepProgress();
				MyGUIStatus.SetStatusInfo( "Processing:\r\n\r\n" +first);

				//now loop over all remaining mp3 objects
				for( int j=i+1; j<files.Length; j++)
				{
					//make sure we are still supposed to keep going
					if( !KeepGoing )
					{
						MyGUIStatus.SetDone(totalFiles, totalMatches, (Environment.TickCount-totalTime)/1000 );
						myStream.Flush();
						myStream.Close();
						return;
					}

					//make sure the file is valid
					if( null == files[j] )
						continue;

					//construct the second mp3 object
					second = ShellID3TagReader.ReadID3Tags(files[j].FullName);

					//file is valid, compare them
					result = CompareFiles(first,second);

					//check the result type
					if( 2 == result )
					{
						myStream.WriteLine("============================");
						myStream.WriteLine(second);
						//delete the first file
						myStream.WriteLine("Deleted---------------------");
						myStream.WriteLine(first);
						myStream.WriteLine("============================\r\n");

						files[i].Delete();
						files[i] = null;
						totalMatches ++;
						break;
					}
					else if( 1 == result)
					{
						myStream.WriteLine("============================");
						myStream.WriteLine(first);
						//delete the second file
						myStream.WriteLine("Deleted---------------");
						myStream.WriteLine(second);
						myStream.WriteLine("============================\r\n");

						files[j].Delete();
						files[j] = null;
						totalMatches++;
						continue;
					}
				} //end for j

				//check to see if we deleted the first file
				if( 1 == result )
					continue;
			}

			MyGUIStatus.SetDone(totalFiles, totalMatches, (Environment.TickCount-totalTime)/1000 );
			myStream.Flush();
			myStream.Close();
		}

		private int GetUserInputOnMatch(MP3File first, MP3File second, String matchStr)
		{
			GUIMatchDialog myMatchD = new GUIMatchDialog();
			myMatchD.SetFile1(first.ToString());
			myMatchD.SetFile2(second.ToString());
			myMatchD.SetMatchTypeDesc(matchStr);
			if( System.Windows.Forms.DialogResult.OK == myMatchD.ShowDialog() )
				return myMatchD.GetResult();
			else
				return -1;
		}

		private int CompareFiles(MP3File first, MP3File second)
		{
			int ret;

			//compare the filenames
			if( -1 != (ret = CompareFileName(first, second)) )
			{
				if( MyConfig.DelPromptFileName )
					return GetUserInputOnMatch(first,second,"Match found on the filename");
				else
					return ret;
			}

			//compare the filesize
			if( -1 != (ret = CompareFileSize(first, second)) )
			{
				if( MyConfig.DelPromptFileSize )
					return GetUserInputOnMatch(first,second, "Match found on the filesize");
				else
					return ret;
			}

			//compare the song title
			if( -1 != (ret = CompareSongTitle(first, second)) )
			{
				if( MyConfig.DelPromptSongTitle )
					return GetUserInputOnMatch(first,second, "Match found on the song title");
				else
					return ret;
			}
			
			//compare the album and track
			if( -1 != (ret = CompareAlbumTrack(first, second)) )
			{
				if( MyConfig.DelPromptAlbumTrack )
					return GetUserInputOnMatch(first,second, "Match found on album and track");
				else
					return ret;
			}
			
			//compare the filename and see if its similar
			if( -1 != (ret = CompareSimilarFileNames(first, second)) )
			{
				if( MyConfig.DelPromptSimilar )
					return GetUserInputOnMatch(first,second, "Match found on similar file names");
				else
					return ret;
			}
			
			//the two files are different
			return 0;
		}

		private int CompareFileName(MP3File first, MP3File second)
		{
			String firstFileName;
			String secondFileName;

			firstFileName = CleanFileName(first.FileName);
			secondFileName = CleanFileName(second.FileName);

			//check if they have the same filename (regardless of case)
			if( 0 == firstFileName.CompareTo( secondFileName ) )
			{
				//their file names are the same
				//keep the larger one
				if( first.FileSize > second.FileSize )
					return 1;
				else
					return 2;
			}
			return -1;
		}


		private int CompareFileSize(MP3File first, MP3File second)
		{
			//now check if the filesize is the same, if so then delete the second
			if( first.FileSize == second.FileSize )
			{
				//the filesizes are the same, probably not a coincidence
				//mark the second one for deletion
				return 2;
			}
			return -1;
		}


		private int CompareSongTitle(MP3File first, MP3File second)
		{
			//make sure they both have a song title (ie not blank)
			if( (first.SongTitle != "") && (second.SongTitle != "") )
			{
				//finally, check the song titles
				if( 0 == first.SongTitle.ToLower().CompareTo( second.SongTitle.ToLower() ))
				{
					//the song titles are identical
					//keep the larger one
					if( first.FileSize > second.FileSize )
						return 1;
					else
						return 2;
				}
			}
			return -1;
		}


		private int CompareAlbumTrack(MP3File first, MP3File second)
		{
			//now check to see if the album and track numbers are the same
			if( (first.AlbumName != "") && (second.AlbumName != "") )
			{
				if(0 == first.AlbumName.ToLower().CompareTo(second.AlbumName.ToLower()) )
				{
					//album names are the same
					//check if the track numbers are the same
					if( (first.TrackNumber!=0) && (first.TrackNumber == second.TrackNumber) )
					{
						//keep the larger one
						if( first.FileSize > second.FileSize )
							return 1;
						else
							return 2;
					}
				}
			}
			return -1;
		}


		private int CompareSimilarFileNames(MP3File first, MP3File second)
		{
			//now for the finalle, check to see how similar the titles are
			if( true == AreStringsSimilar( first.FileName, second.FileName) )
			{
				if( first.FileSize > second.FileSize )
					return 1;
				else
					return 2;
			}
			return -1;
		}


		private bool AreStringsSimilar( String first, String second )
		{
			string[] firstToks = first.Split(new char[1]{' '});
			string[] secondToks = second.Split(new char[1]{' '});
			int inCommon = 0;
			int lastInSecondToks = -1;
			int percentCommon;

			for(int i=0; i<firstToks.Length; i++)
			{
				//the artist name usually goes first
				//i.e. "Frank Sinatra - Singing in the rain"
				if( 0 == firstToks[i].CompareTo("-"))
				{
					//check to see if we have more than one word past the "-"
					if( (i+2) >= firstToks.Length )
					{
						//we dont have enough words to do a proper comparision
						return false;
					}
				}

				for(int j=lastInSecondToks+1; j<secondToks.Length; j++)
				{
					if(0 == firstToks[i].ToLower().CompareTo( secondToks[j].ToLower()) )
					{
						lastInSecondToks = j;
						inCommon++;
						break;
					}
				}
			}
	
			//get the larger larger string list
			percentCommon = (firstToks.Length>secondToks.Length)?firstToks.Length:secondToks.Length;
			//now count how many we have in common between the two string lists,
			//relative to the size of the larger string list
			percentCommon = inCommon*100/percentCommon;
			//if we have greater than 70% match, they are the same
			if( 65 <= percentCommon )
			{
				return true;
			}
			return false;
		}

		private String CleanFileName( String filename )
		{
			String ret;
			int index;
			int index2;

			//make it lower case
			ret = filename.ToLower();
			
			//first get ride of the file extension (even .gif.mp3 compound names)
			index = ret.IndexOf(".");
			ret = ret.Substring(0,index);

			//now get rid of all the "copy of " parts
			index = ret.IndexOf("copy of ");
			while( -1 != index )
			{	
				if(0 == index)
				{
					ret = ret.Substring(8);
				}
				else
				{
					ret = ret.Substring(0,index) + ret.Substring(index+8);
				}
				index = ret.IndexOf("copy of ");
			}

			//now get rid of any (#) in the name
			index = ret.IndexOf("(");
			while( -1 != index )
			{	
				//now see if there is an ) after the (
				index2 = ret.IndexOf(")",index);
				if(-1 == index2)
				{
					break;
				}
				else
				{
					ret = ret.Substring(0,index) + ret.Substring(index2+1);
				}
				index = ret.IndexOf("(");
			}


			//now get rid of any [#] in the name
			index = ret.IndexOf("[");
			while( -1 != index )
			{	
				//now see if there is an ] after the [
				index2 = ret.IndexOf("]", index);
				if(-1 == index2)
				{
					break;
				}
				else
				{
					ret = ret.Substring(0,index) + ret.Substring(index2+1);
				}
				index = ret.IndexOf("[");
			}

			//finally, get rid of any trailing whitespace
			ret = ret.Trim();
			return ret;
		}
	}
}
